@@ -24,12 +24,12 @@ from sqlalchemy.orm import backref
 from sqlalchemy.orm.collections import attribute_mapped_collection
 from sqlalchemy.orm import exc
 
+from neutron.db import api as db_api
 from neutron.db import common_db_mixin
 from neutron.db import models_v2
 from neutron_lib.db import constants as db_const
 from neutron_lib.db import model_base
 
-from networking_sfc._i18n import _LI
 from networking_sfc.db import flowclassifier_db as fc_db
 from networking_sfc.extensions import flowclassifier as ext_fc
 from networking_sfc.extensions import sfc as ext_sfc
@@ -39,6 +39,7 @@ LOG = logging.getLogger(__name__)
 
 UUID_LEN = 36
 PARAM_LEN = 255
+VAR_MAX_LEN = 1024
 
 
 class ChainParameter(model_base.BASEV2):
@@ -67,7 +68,7 @@ class PortPairGroupParam(model_base.BASEV2):
     """Represents a port pair group parameter."""
     __tablename__ = 'sfc_port_pair_group_params'
     keyword = sa.Column(sa.String(PARAM_LEN), primary_key=True)
-    value = sa.Column(sa.String(PARAM_LEN))
+    value = sa.Column(sa.String(VAR_MAX_LEN))
     pair_group_id = sa.Column(
         sa.String(UUID_LEN),
         sa.ForeignKey('sfc_port_pair_groups.id', ondelete='CASCADE'),
@@ -210,7 +211,7 @@ class SfcDbPlugin(
         return self._fields(res, fields)
 
     def _validate_port_pair_groups(self, context, pg_ids, pc_id=None):
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.reader.using(context):
             for pg_id in pg_ids:
                 self._get_port_pair_group(context, pg_id)
             query = self._model_query(context, PortChain)
@@ -226,7 +227,7 @@ class SfcDbPlugin(
                         port_pair_groups=pg_ids, port_chain=port_chain_db.id)
 
     def _validate_flow_classifiers(self, context, fc_ids, pc_id=None):
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.reader.using(context):
             fcs = [
                 self._get_flow_classifier(context, fc_id)
                 for fc_id in fc_ids
@@ -262,7 +263,7 @@ class SfcDbPlugin(
     def _setup_chain_group_associations(
         self, context, port_chain, pg_ids
     ):
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.reader.using(context):
             chain_group_associations = []
             for pg_id in pg_ids:
                 query = self._model_query(context, ChainGroupAssoc)
@@ -279,7 +280,7 @@ class SfcDbPlugin(
     def _setup_chain_classifier_associations(
         self, context, port_chain, fc_ids
     ):
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.reader.using(context):
             chain_classifier_associations = []
             for fc_id in fc_ids:
                 query = self._model_query(context, ChainClassifierAssoc)
@@ -301,7 +302,7 @@ class SfcDbPlugin(
         pc = port_chain['port_chain']
         project_id = pc['project_id']
         chain_id = pc['chain_id']
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             chain_parameters = {
                 key: ChainParameter(keyword=key, value=jsonutils.dumps(val))
                 for key, val in pc['chain_parameters'].items()}
@@ -376,16 +377,16 @@ class SfcDbPlugin(
     @log_helpers.log_method_call
     def delete_port_chain(self, context, id):
         try:
-            with context.session.begin(subtransactions=True):
+            with db_api.context_manager.writer.using(context):
                 pc = self._get_port_chain(context, id)
                 context.session.delete(pc)
         except ext_sfc.PortChainNotFound:
-            LOG.info(_LI("Deleting a non-existing port chain."))
+            LOG.info("Deleting a non-existing port chain.")
 
     @log_helpers.log_method_call
     def update_port_chain(self, context, id, port_chain):
         pc = port_chain['port_chain']
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             pc_db = self._get_port_chain(context, id)
             for k, v in pc.items():
                 if k == 'flow_classifiers':
@@ -438,7 +439,7 @@ class SfcDbPlugin(
         """Create a port pair."""
         pp = port_pair['port_pair']
         project_id = pp['project_id']
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             query = self._model_query(context, PortPair)
             pp_in_use = query.filter_by(
                 ingress=pp['ingress'], egress=pp['egress']
@@ -509,7 +510,7 @@ class SfcDbPlugin(
     @log_helpers.log_method_call
     def update_port_pair(self, context, id, port_pair):
         new_pp = port_pair['port_pair']
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             old_pp = self._get_port_pair(context, id)
             old_pp.update(new_pp)
             return self._make_port_pair_dict(old_pp)
@@ -517,13 +518,13 @@ class SfcDbPlugin(
     @log_helpers.log_method_call
     def delete_port_pair(self, context, id):
         try:
-            with context.session.begin(subtransactions=True):
+            with db_api.context_manager.writer.using(context):
                 pp = self._get_port_pair(context, id)
                 if pp.portpairgroup_id:
                     raise ext_sfc.PortPairInUse(id=id)
                 context.session.delete(pp)
         except ext_sfc.PortPairNotFound:
-            LOG.info(_LI("Deleting a non-existing port pair."))
+            LOG.info("Deleting a non-existing port pair.")
 
     def _make_port_pair_group_dict(self, port_pair_group, fields=None):
         res = {
@@ -542,18 +543,35 @@ class SfcDbPlugin(
 
         return self._fields(res, fields)
 
+    def _validate_pps_in_ppg(self, portpairs_list, id=None):
+        first_check = True
+        correlation = None
+        for portpair in portpairs_list:
+            sfparams = portpair.service_function_parameters
+            pp_corr = sfparams['correlation']
+            if first_check:
+                first_check = False
+                correlation = pp_corr.value
+            if pp_corr.value != correlation:
+                # don't include PPs of different correlations
+                raise ext_sfc.InconsistentCorrelations()
+            if (
+                portpair.portpairgroup_id and
+                portpair.portpairgroup_id != id
+            ):
+                # don't include PPs included by other PPGs
+                raise ext_sfc.PortPairInUse(id=portpair.id)
+
     @log_helpers.log_method_call
     def create_port_pair_group(self, context, port_pair_group):
         """Create a port pair group."""
         pg = port_pair_group['port_pair_group']
         project_id = pg['project_id']
 
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             portpairs_list = [self._get_port_pair(context, pp_id)
                               for pp_id in pg['port_pairs']]
-            for portpair in portpairs_list:
-                if portpair.portpairgroup_id:
-                    raise ext_sfc.PortPairInUse(id=portpair.id)
+            self._validate_pps_in_ppg(portpairs_list)
             port_pair_group_parameters = {
                 key: PortPairGroupParam(
                     keyword=key, value=jsonutils.dumps(val))
@@ -623,16 +641,10 @@ class SfcDbPlugin(
     def update_port_pair_group(self, context, id, port_pair_group):
         new_pg = port_pair_group['port_pair_group']
 
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             portpairs_list = [self._get_port_pair(context, pp_id)
                               for pp_id in new_pg.get('port_pairs', [])]
-            for portpair in portpairs_list:
-                if (
-                    portpair.portpairgroup_id and
-                    portpair.portpairgroup_id != id
-                ):
-                    raise ext_sfc.PortPairInUse(id=portpair.id)
-
+            self._validate_pps_in_ppg(portpairs_list, id)
             old_pg = self._get_port_pair_group(context, id)
             for k, v in new_pg.items():
                 if k == 'port_pairs':
@@ -649,10 +661,10 @@ class SfcDbPlugin(
     @log_helpers.log_method_call
     def delete_port_pair_group(self, context, id):
         try:
-            with context.session.begin(subtransactions=True):
+            with db_api.context_manager.writer.using(context):
                 pg = self._get_port_pair_group(context, id)
                 if pg.chain_group_associations:
                     raise ext_sfc.PortPairGroupInUse(id=id)
                 context.session.delete(pg)
         except ext_sfc.PortPairGroupNotFound:
-            LOG.info(_LI("Deleting a non-existing port pair group."))
+            LOG.info("Deleting a non-existing port pair group.")
