@@ -13,19 +13,18 @@
 #    under the License.
 
 import copy
-import mock
-from webob import exc
-import webtest
 
-from oslo_utils import uuidutils
+import mock
 
 from neutron.api.v2 import resource as api_res_log
 from neutron.common import config as cfg
 from neutron import manager
 from neutron.notifiers import nova as nova_log
-
 from neutron.tests.unit.api.v2 import test_base as test_api_v2
 from neutron.tests.unit.extensions import base as test_api_v2_extension
+from oslo_utils import uuidutils
+from webob import exc
+import webtest
 
 from networking_sfc.extensions import sfc as sfc_ext
 
@@ -347,7 +346,9 @@ class SfcExtensionTestCase(test_api_v2_extension.ExtensionTestCase):
             'project_id': port_pair_group['project_id'],
             'port_pair_group_parameters': port_pair_group.get(
                 'port_pair_group_parameters'
-            ) or {'lb_fields': []}
+            ) or {'lb_fields': [],
+                  'ppg_n_tuple_mapping': {'ingress_n_tuple': {},
+                                          'egress_n_tuple': {}}}
         }}
         if port_pair_group.get('group_id'):
             ret['port_pair_group']['group_id'] = port_pair_group['group_id']
@@ -493,6 +494,25 @@ class SfcExtensionTestCase(test_api_v2_extension.ExtensionTestCase):
             self.serialize(data),
             content_type='application/%s' % self.fmt)
 
+    def test_create_port_pair_group_invalid_ppg_n_tuple_mapping_key(self):
+        tenant_id = _uuid()
+        data = {'port_pair_group': {
+            'port_pairs': [_uuid()],
+            'port_pair_group_parameters': {
+                'ppg_n_tuple_mapping': {
+                    'ingress_n_tuple': {'sssource_ip_prefix': None},
+                    'egress_n_tuple': {'protool': None}
+                }
+            },
+            'tenant_id': tenant_id, 'project_id': tenant_id
+        }}
+        self.assertRaises(
+            webtest.app.AppError,
+            self.api.post,
+            _get_path(PORT_PAIR_GROUP_PATH, fmt=self.fmt),
+            self.serialize(data),
+            content_type='application/%s' % self.fmt)
+
     def test_create_port_pair_group_nonuuid_port_pairs(self):
         tenant_id = _uuid()
         data = {'port_pair_group': {
